@@ -22,13 +22,13 @@ from adcircpy.mesh.figures import _figure as _fig
 
 class EuclideanMesh2D:
     def __init__(
-          self,
-          coords,
-          triangles=None,
-          quads=None,
-          values=None,
-          crs=None,
-          description=None,
+            self,
+            coords,
+            triangles=None,
+            quads=None,
+            values=None,
+            crs=None,
+            description=None,
     ):
         self._coords = coords
         self._triangles = triangles
@@ -98,50 +98,12 @@ class EuclideanMesh2D:
                 return np.vstack([x, y]).T
         return np.vstack([self.x, self.y]).T
 
-    @property
-    def xy(self):
-        return self.coords
-
     def get_x(self, crs=None):
         return self.get_xy(crs)[:, 0]
 
-    @property
-    def x(self):
-        return self.coords[:, 0]
-
     def get_y(self, crs=None):
         return self.get_xy(crs)[:, 1]
 
-    @property
-    def y(self):
-        return self.coords[:, 1]
-
-    @property
-    @lru_cache(maxsize=None)
-    def coords(self):
-        return np.array([coord for coord in self._coords.values()])
-
-    @property
-    @lru_cache(maxsize=None)
-    def coords_id(self):
-        return self._coords.keys()
-
-    @property
-    def _coords(self):
-        return self.__coords
-
-    @_coords.setter
-    def _coords(self, coords):
-        msg = 'coord argument must be a dictionary of the form ' \
-              '\\{coord_id:  (x, y)\\}'
-        assert isinstance(coords, Mapping), msg
-        for coord in coords.values():
-            assert len(coord) == 2, msg
-            assert isinstance(coord[0], (float, int)), msg
-            assert isinstance(coord[1], (float, int)), msg
-        self.__coords = coords
-        type(self).coords.fget.cache_clear()
-
     def get_extent(self, crs=None):
         xy = self.get_xy(crs)
         return (np.min(xy[:, 0]), np.max(xy[:, 0]),
@@ -151,10 +113,6 @@ class EuclideanMesh2D:
         xmin, xmax, ymin, ymax = self.get_extent(crs)
         return Bbox([[xmin, ymin], [xmax, ymax]])
 
-    @property
-    def bbox(self):
-        return self.get_bbox()
-
     def add_attribute(self, name, **properties):
         if self.has_attribute(name):
             raise AttributeError('Non-unique attribute name: '
@@ -191,7 +149,6 @@ class EuclideanMesh2D:
         return self._attributes[name]['values']
 
     def get_attribute_properties(self, name):
-        # TODO this is never used; is it meant to return the non-values dictionary?
         if not self.has_attribute(name):
             raise AttributeError(f'Attribute {name} not set.')
         return self._attributes[name]['properties']
@@ -206,38 +163,141 @@ class EuclideanMesh2D:
             raise AttributeError('Cannot remove attribute: '
                                  'attribute does not exist.')
 
+    def get_node_id(self, index):
+        return self.node_id[index]
+
+    def get_node_index(self, id):
+        return self.node_index[id]
+
+    def get_element_id(self, index):
+        return self.element_id[index]
+
+    def get_element_index(self, id):
+        return self.element_index[id]
+
+    def transform_to(self, dst_crs):
+        dst_crs = CRS.from_user_input(dst_crs)
+        if self.crs != dst_crs:
+            transformer = Transformer.from_crs(self.crs, dst_crs,
+                                               always_xy=True)
+            xy = list(zip(*transformer.transform(self.x, self.y)))
+            ids = list(self._coords.keys())
+            self._coords = {ids[i]: coord for i, coord in enumerate(xy)}
+            self._crs = dst_crs
+
+    # plotting functions
+    @_fig
+    def tricontourf(self, axes=None, show=True, figsize=None, **kwargs):
+        if len(self.tria3) > 0:
+            axes.tricontourf(self.triangulation, self.values, **kwargs)
+        return axes
+
+    @_fig
+    def tripcolor(self, axes=None, show=True, figsize=None, **kwargs):
+        if len(self.tria3) > 0:
+            axes.tripcolor(self.triangulation, self.values, **kwargs)
+        return axes
+
+    @_fig
+    def triplot(
+            self,
+            axes=None,
+            show=False,
+            figsize=None,
+            linewidth=0.07,
+            color='black',
+            **kwargs
+    ):
+        if len(self.triangles) > 0:
+            kwargs.update({'linewidth': linewidth})
+            kwargs.update({'color': color})
+            axes.triplot(self.triangulation, **kwargs)
+        if show:
+            pyplot.show()
+        return axes
+
+    @_fig
+    def quadplot(
+            self,
+            axes=None,
+            show=False,
+            figsize=None,
+            facecolor='none',
+            edgecolor='k',
+            linewidth=0.07,
+            **kwargs
+    ):
+        if len(self.quads) > 0:
+            pc = PolyCollection(
+                self.coords[self.quads],
+                facecolor=facecolor,
+                edgecolor=edgecolor,
+                linewidth=0.07,
+            )
+            axes.add_collection(pc)
+        if show:
+            pyplot.show()
+        return axes
+
+    @_fig
+    def quadface(
+            self,
+            axes=None,
+            show=False,
+            figsize=None,
+            **kwargs
+    ):
+        if len(self.quad4) > 0:
+            pc = PolyCollection(
+                self.coords[self.quad4],
+                **kwargs
+            )
+            quad_value = np.mean(self.values[self.quad4], axis=1)
+            pc.set_array(quad_value)
+            axes.add_collection(pc)
+        if show:
+            pyplot.show()
+        return axes
+
+    @_fig
+    def plot_wireframe(self, axes=None, show=False, **kwargs):
+        axes = self.triplot(axes=axes, **kwargs)
+        axes = self.quadplot(axes=axes, **kwargs)
+        if show:
+            pyplot.show()
+        return axes
+
+    @property
+    def xy(self):
+        return self.coords
+
+    @property
+    def x(self):
+        return self.coords[:, 0]
+
+    @property
+    def y(self):
+        return self.coords[:, 1]
+
     @property
     @lru_cache(maxsize=None)
-    def _attributes(self):
-        return {}
+    def coords(self):
+        return np.array([coord for coord in self._coords.values()])
+
+    @property
+    @lru_cache(maxsize=None)
+    def coords_id(self):
+        return self._coords.keys()
+
+    @property
+    def bbox(self):
+        return self.get_bbox()
 
     @property
     @lru_cache(maxsize=None)
     def values(self):
         return self._values
 
-    @property
-    def _values(self):
-        return self.__values
-
-    @_values.setter
-    def _values(self, values):
-        if values is None:
-            values = []
-        values = np.asarray(values)
-        if len(values) > 0:
-            if len(values.shape) in [1, 2]:
-                assert values.shape[0] == self.coords.shape[0]
-            elif len(values.shape) == 3:
-                assert values.shape[1] == self.coords.shape[0]
-            else:
-                raise Exception(f'input values has invalid shape: '
-                                f'{values.shape}')
-        else:
-            values = np.full(self.coords.shape[0], np.nan)
-        self.__values = values
-        # type(self).values.fget.cache_clear()
-
     @property
     @lru_cache(maxsize=None)
     def nodes(self):
@@ -245,21 +305,136 @@ class EuclideanMesh2D:
 
     @property
     @lru_cache(maxsize=None)
-    def _nodes(self):
-        return {id: ((x, y), -self.values[i])
-                for i, (id, (x, y)) in enumerate(self._coords.items())}
+    def triangles_id(self):
+        return self._triangles.keys()
 
-    def get_node_id(self, index):
-        return self.node_id[index]
+    @property
+    @lru_cache(maxsize=None)
+    def tria3(self):
+        return np.array([list(map(self.get_node_index, index))
+                         for index in self._triangles.values()])
+
+    @property
+    def triangulation(self):
+        if len(self.tria3) > 0:
+            return Triangulation(self.x, self.y, self.tria3)
+
+    @property
+    def triangles(self):
+        return self.tria3
+
+    @property
+    @lru_cache(maxsize=None)
+    def quads_id(self):
+        return self._quads.keys()
+
+    @property
+    @lru_cache(maxsize=None)
+    def quad4(self):
+        return np.array([list(map(self.get_node_index, index))
+                         for index in self._quads.values()])
+
+    @property
+    def quads(self):
+        return self.quad4
+
+    @property
+    @lru_cache(maxsize=None)
+    def index_ring_collection(self):
+        # find boundary edges using triangulation neighbors table,
+        # see: https://stackoverflow.com/a/23073229/7432462
+        boundary_edges = list()
+        tri = self.triangulation
+        idxs = np.vstack(
+            list(np.where(tri.neighbors == -1))).T
+        for i, j in idxs:
+            boundary_edges.append(
+                (int(tri.triangles[i, j]),
+                 int(tri.triangles[i, (j + 1) % 3])))
+        index_ring_collection = self.sort_edges(boundary_edges)
+        # sort index_rings into corresponding "polygons"
+        areas = list()
+        vertices = self.coords
+        for index_ring in index_ring_collection:
+            e0, e1 = [list(t) for t in zip(*index_ring)]
+            areas.append(float(Polygon(vertices[e0, :]).area))
+
+        # maximum area must be main mesh
+        idx = areas.index(np.max(areas))
+        exterior = index_ring_collection.pop(idx)
+        areas.pop(idx)
+        _id = 0
+        _index_ring_collection = dict()
+        _index_ring_collection[_id] = {
+            'exterior': np.asarray(exterior),
+            'interiors': []
+        }
+        e0, e1 = [list(t) for t in zip(*exterior)]
+        path = Path(vertices[e0 + [e0[0]], :], closed=True)
+        while len(index_ring_collection) > 0:
+            # find all internal rings
+            potential_interiors = list()
+            for i, index_ring in enumerate(index_ring_collection):
+                e0, e1 = [list(t) for t in zip(*index_ring)]
+                if path.contains_point(vertices[e0[0], :]):
+                    potential_interiors.append(i)
+            # filter out nested rings
+            real_interiors = list()
+            for i, p_interior in reversed(
+                    list(enumerate(potential_interiors))):
+                _p_interior = index_ring_collection[p_interior]
+                check = [index_ring_collection[_]
+                         for j, _ in
+                         reversed(list(enumerate(potential_interiors)))
+                         if i != j]
+                has_parent = False
+                for _path in check:
+                    e0, e1 = [list(t) for t in zip(*_path)]
+                    _path = Path(vertices[e0 + [e0[0]], :], closed=True)
+                    if _path.contains_point(vertices[_p_interior[0][0], :]):
+                        has_parent = True
+                if not has_parent:
+                    real_interiors.append(p_interior)
+            # pop real rings from collection
+            for i in reversed(sorted(real_interiors)):
+                _index_ring_collection[_id]['interiors'].append(
+                    np.asarray(index_ring_collection.pop(i)))
+                areas.pop(i)
+            # if no internal rings found, initialize next polygon
+            if len(index_ring_collection) > 0:
+                idx = areas.index(np.max(areas))
+                exterior = index_ring_collection.pop(idx)
+                areas.pop(idx)
+                _id += 1
+                _index_ring_collection[_id] = {
+                    'exterior': np.asarray(exterior),
+                    'interiors': []
+                }
+                e0, e1 = [list(t) for t in zip(*exterior)]
+                path = Path(vertices[e0 + [e0[0]], :], closed=True)
+        return _index_ring_collection
+
+    @property
+    @lru_cache(maxsize=None)
+    def outer_ring_collection(self):
+        outer_ring_collection = defaultdict()
+        for key, ring in self.index_ring_collection.items():
+            outer_ring_collection[key] = ring['exterior']
+        return outer_ring_collection
+
+    @property
+    @lru_cache(maxsize=None)
+    def inner_ring_collection(self):
+        inner_ring_collection = defaultdict()
+        for key, rings in self.index_ring_collection.items():
+            inner_ring_collection[key] = rings['interiors']
+        return inner_ring_collection
 
     @property
     @lru_cache(maxsize=None)
     def node_id(self):
         return {index: id for index, id in enumerate(self._coords)}
 
-    def get_node_index(self, id):
-        return self.node_index[id]
-
     @property
     @lru_cache(maxsize=None)
     def node_index(self):
@@ -291,12 +466,6 @@ class EuclideanMesh2D:
                 )
         return node_distances
 
-    def get_element_index(self, id):
-        return self.element_index[id]
-
-    def get_element_id(self, index):
-        return self.element_id[index]
-
     @property
     @lru_cache(maxsize=None)
     def elements(self):
@@ -313,19 +482,12 @@ class EuclideanMesh2D:
         return {id: index for index, id in enumerate(self._elements)}
 
     @property
-    @lru_cache(maxsize=None)
-    def _elements(self):
-        elements_id = list()
-        elements_id.extend(list(self._triangles.keys()))
-        elements_id.extend(list(self._quads.keys()))
-        elements_id = range(1, len(elements_id) + 1) \
-            if len(set(elements_id)) != len(elements_id) else elements_id
-        elements = list()
-        elements.extend(list(self._triangles.values()))
-        elements.extend(list(self._quads.values()))
-        elements = {
-            elements_id[i]: indexes for i, indexes in enumerate(elements)}
-        return elements
+    def grd(self):
+        return grd.string(self._grd)
+
+    @property
+    def sms2dm(self):
+        return sms2dm.string(self._sms2dm)
 
     @property
     def description(self):
@@ -335,31 +497,10 @@ class EuclideanMesh2D:
     def description(self, description):
         self._description = description
 
-    @property
-    def _description(self):
-        return self.__description
-
-    @_description.setter
-    def _description(self, description):
-        if description is None:
-            description = uuid.uuid4().hex[:8]
-        assert isinstance(description, str)
-        self.__description = description
-
     @property
     def crs(self):
         return self._crs
 
-    @property
-    def _crs(self):
-        return self.__crs
-
-    @_crs.setter
-    def _crs(self, crs):
-        if crs is not None:
-            crs = CRS.from_user_input(crs)
-        self.__crs = crs
-
     @property
     def proj(self):
         return Proj(self.crs)
@@ -368,215 +509,6 @@ class EuclideanMesh2D:
     def srs(self):
         return self.proj.srs
 
-    def transform_to(self, dst_crs):
-        dst_crs = CRS.from_user_input(dst_crs)
-        if self.crs != dst_crs:
-            transformer = Transformer.from_crs(self.crs, dst_crs,
-                                               always_xy=True)
-            xy = list(zip(*transformer.transform(self.x, self.y)))
-            ids = list(self._coords.keys())
-            self._coords = {ids[i]: coord for i, coord in enumerate(xy)}
-            self._crs = dst_crs
-
-    @property
-    def grd(self):
-        return grd.string(self._grd)
-
-    @property
-    @lru_cache(maxsize=None)
-    def _grd(self):
-        description = self.description
-        if self.crs is not None and self.crs.srs not in description:
-            description += f'; {self.crs.srs}'
-        return {
-            'nodes'      : self._nodes,
-            'elements'   : self._elements,
-            'description': description,
-        }
-
-    @property
-    def sms2dm(self):
-        return sms2dm.string(self._sms2dm)
-
-    @property
-    @lru_cache(maxsize=None)
-    def _sms2dm(self):
-        description = self.description
-        if self.crs is not None:
-            description += f'; {self.crs.srs}'
-        return {
-            'ND' : self._nodes,
-            'E3T': self._triangles,
-            'E4Q': self._quads,
-        }
-
-    @property
-    @lru_cache(maxsize=None)
-    def triangles_id(self):
-        return self._triangles.keys()
-
-    @property
-    @lru_cache(maxsize=None)
-    def tria3(self):
-        return np.array([list(map(self.get_node_index, index))
-                         for index in self._triangles.values()])
-
-    @property
-    def triangulation(self):
-        if len(self.tria3) > 0:
-            return Triangulation(self.x, self.y, self.tria3)
-
-    @property
-    def triangles(self):
-        return self.tria3
-
-    @property
-    def _triangles(self):
-        return self.__triangles
-
-    @_triangles.setter
-    def _triangles(self, triangles):
-        if triangles is None:
-            triangles = {}
-        self._certify_input_geom('triangles', triangles)
-        self.__triangles = triangles
-
-    @property
-    @lru_cache(maxsize=None)
-    def quads_id(self):
-        return self._quads.keys()
-
-    @property
-    @lru_cache(maxsize=None)
-    def quad4(self):
-        return np.array([list(map(self.get_node_index, index))
-                         for index in self._quads.values()])
-
-    @property
-    def quads(self):
-        return self.quad4
-
-    @property
-    def _quads(self):
-        return self.__quads
-
-    @_quads.setter
-    def _quads(self, quads):
-        if quads is None:
-            quads = {}
-        self._certify_input_geom('quads', quads)
-        self.__quads = quads
-
-    @property
-    @lru_cache(maxsize=None)
-    def index_ring_collection(self):
-        # find boundary edges using triangulation neighbors table,
-        # see: https://stackoverflow.com/a/23073229/7432462
-        boundary_edges = list()
-        tri = self.triangulation
-        idxs = np.vstack(
-            list(np.where(tri.neighbors == -1))).T
-        for i, j in idxs:
-            boundary_edges.append(
-                (int(tri.triangles[i, j]),
-                 int(tri.triangles[i, (j + 1) % 3])))
-        index_ring_collection = self.sort_edges(boundary_edges)
-        # sort index_rings into corresponding "polygons"
-        areas = list()
-        vertices = self.coords
-        for index_ring in index_ring_collection:
-            e0, e1 = [list(t) for t in zip(*index_ring)]
-            areas.append(float(Polygon(vertices[e0, :]).area))
-
-        # maximum area must be main mesh
-        idx = areas.index(np.max(areas))
-        exterior = index_ring_collection.pop(idx)
-        areas.pop(idx)
-        _id = 0
-        _index_ring_collection = dict()
-        _index_ring_collection[_id] = {
-            'exterior' : np.asarray(exterior),
-            'interiors': []
-        }
-        e0, e1 = [list(t) for t in zip(*exterior)]
-        path = Path(vertices[e0 + [e0[0]], :], closed=True)
-        while len(index_ring_collection) > 0:
-            # find all internal rings
-            potential_interiors = list()
-            for i, index_ring in enumerate(index_ring_collection):
-                e0, e1 = [list(t) for t in zip(*index_ring)]
-                if path.contains_point(vertices[e0[0], :]):
-                    potential_interiors.append(i)
-            # filter out nested rings
-            real_interiors = list()
-            for i, p_interior in reversed(list(enumerate(potential_interiors))):
-                _p_interior = index_ring_collection[p_interior]
-                check = [index_ring_collection[_]
-                         for j, _ in
-                         reversed(list(enumerate(potential_interiors)))
-                         if i != j]
-                has_parent = False
-                for _path in check:
-                    e0, e1 = [list(t) for t in zip(*_path)]
-                    _path = Path(vertices[e0 + [e0[0]], :], closed=True)
-                    if _path.contains_point(vertices[_p_interior[0][0], :]):
-                        has_parent = True
-                if not has_parent:
-                    real_interiors.append(p_interior)
-            # pop real rings from collection
-            for i in reversed(sorted(real_interiors)):
-                _index_ring_collection[_id]['interiors'].append(
-                    np.asarray(index_ring_collection.pop(i)))
-                areas.pop(i)
-            # if no internal rings found, initialize next polygon
-            if len(index_ring_collection) > 0:
-                idx = areas.index(np.max(areas))
-                exterior = index_ring_collection.pop(idx)
-                areas.pop(idx)
-                _id += 1
-                _index_ring_collection[_id] = {
-                    'exterior' : np.asarray(exterior),
-                    'interiors': []
-                }
-                e0, e1 = [list(t) for t in zip(*exterior)]
-                path = Path(vertices[e0 + [e0[0]], :], closed=True)
-        return _index_ring_collection
-
-    @property
-    @lru_cache(maxsize=None)
-    def outer_ring_collection(self):
-        outer_ring_collection = defaultdict()
-        for key, ring in self.index_ring_collection.items():
-            outer_ring_collection[key] = ring['exterior']
-        return outer_ring_collection
-
-    @property
-    @lru_cache(maxsize=None)
-    def inner_ring_collection(self):
-        inner_ring_collection = defaultdict()
-        for key, rings in self.index_ring_collection.items():
-            inner_ring_collection[key] = rings['interiors']
-        return inner_ring_collection
-
-    def _certify_input_geom(self, geom_type, geom):
-        geom_types = {
-            'triangles': 3,
-            'quads'    : 4,
-        }
-        _geom = np.asarray(list(geom.values()))
-        if len(_geom) > 0:
-            assert _geom.shape[1] == geom_types[geom_type], \
-                f'Invalid shape for geom {_geom.shape}.'
-            for IDtags in geom.values():
-                for IDtag in IDtags:
-                    assert IDtag in self.coords_id, \
-                        f'{geom_type} members must be a subset of coords keys'
-
-    @property
-    @lru_cache(maxsize=None)
-    def _logger(self):
-        return logging.getLogger(__name__ + '.' + self.__class__.__name__)
-
     # auxilliary functions
     @staticmethod
     def sort_edges(edges):
@@ -625,88 +557,157 @@ class EuclideanMesh2D:
             area -= vertices[j][0] * vertices[i][1]
         return area / 2.0
 
-    # plotting functions
-    @_fig
-    def tricontourf(self, axes=None, show=True, figsize=None, **kwargs):
-        if len(self.tria3) > 0:
-            axes.tricontourf(self.triangulation, self.values, **kwargs)
-        if show:
-            pyplot.show()
-        return axes
+    def _certify_input_geom(self, geom_type, geom):
+        geom_types = {
+            'triangles': 3,
+            'quads': 4,
+        }
+        _geom = np.asarray(list(geom.values()))
+        if len(_geom) > 0:
+            assert _geom.shape[1] == geom_types[geom_type], \
+                f'Invalid shape for geom {_geom.shape}.'
+            for IDtags in geom.values():
+                for IDtag in IDtags:
+                    assert IDtag in self.coords_id, \
+                        f'{geom_type} members must be a subset of coords keys'
 
-    @_fig
-    def tripcolor(self, axes=None, show=True, figsize=None, **kwargs):
-        if len(self.tria3) > 0:
-            axes.tripcolor(self.triangulation, self.values, **kwargs)
-        if show:
-            pyplot.show()
-        return axes
+    @property
+    @lru_cache(maxsize=None)
+    def _attributes(self):
+        return {}
 
-    @_fig
-    def triplot(
-          self,
-          axes=None,
-          show=False,
-          figsize=None,
-          linewidth=0.07,
-          color='black',
-          **kwargs
-    ):
-        if len(self.triangles) > 0:
-            kwargs.update({'linewidth': linewidth})
-            kwargs.update({'color': color})
-            axes.triplot(self.triangulation, **kwargs)
-        if show:
-            pyplot.show()
-        return axes
+    @property
+    @lru_cache(maxsize=None)
+    def _nodes(self):
+        return {id: ((x, y), -self.values[i])
+                for i, (id, (x, y)) in enumerate(self._coords.items())}
 
-    @_fig
-    def quadplot(
-          self,
-          axes=None,
-          show=False,
-          figsize=None,
-          facecolor='none',
-          edgecolor='k',
-          linewidth=0.07,
-          **kwargs
-    ):
-        if len(self.quads) > 0:
-            pc = PolyCollection(
-                self.coords[self.quads],
-                facecolor=facecolor,
-                edgecolor=edgecolor,
-                linewidth=0.07,
-            )
-            axes.add_collection(pc)
-        if show:
-            pyplot.show()
-        return axes
+    @property
+    def _description(self):
+        return self.__description
 
-    @_fig
-    def quadface(
-          self,
-          axes=None,
-          show=False,
-          figsize=None,
-          **kwargs
-    ):
-        if len(self.quad4) > 0:
-            pc = PolyCollection(
-                self.coords[self.quad4],
-                **kwargs
-            )
-            quad_value = np.mean(self.values[self.quad4], axis=1)
-            pc.set_array(quad_value)
-            axes.add_collection(pc)
-        if show:
-            pyplot.show()
-        return axes
+    @_description.setter
+    def _description(self, description):
+        if description is None:
+            description = uuid.uuid4().hex[:8]
+        assert isinstance(description, str)
+        self.__description = description
 
-    @_fig
-    def plot_wireframe(self, axes=None, show=False, **kwargs):
-        axes = self.triplot(axes=axes, **kwargs)
-        axes = self.quadplot(axes=axes, **kwargs)
-        if show:
-            pyplot.show()
-        return axes
+    @property
+    def _crs(self):
+        return self.__crs
+
+    @_crs.setter
+    def _crs(self, crs):
+        if crs is not None:
+            crs = CRS.from_user_input(crs)
+        self.__crs = crs
+
+    @property
+    def _coords(self):
+        return self.__coords
+
+    @_coords.setter
+    def _coords(self, coords):
+        if isinstance(coords, np.ndarray):
+            coords = {id: tuple(vals) for id, vals in enumerate(coords)}
+        msg = 'coord argument must be a dictionary of the form ' \
+              r'{coord_id:  (x, y)} or a numpy array.'
+        assert isinstance(coords, Mapping), msg
+        for coord in coords.values():
+            assert len(coord) == 2, msg
+            assert isinstance(coord[0], (float, int)), msg
+            assert isinstance(coord[1], (float, int)), msg
+        self.__coords = coords
+
+    @property
+    def _values(self):
+        return self.__values
+
+    @_values.setter
+    def _values(self, values):
+        if values is None:
+            values = []
+        values = np.asarray(values)
+        if len(values) > 0:
+            if len(values.shape) in [1, 2]:
+                assert values.shape[0] == self.coords.shape[0]
+            elif len(values.shape) == 3:
+                assert values.shape[1] == self.coords.shape[0]
+            else:
+                raise Exception(f'input values has invalid shape: '
+                                f'{values.shape}')
+        else:
+            values = np.full(self.coords.shape[0], np.nan)
+        self.__values = values
+        # type(self).values.fget.cache_clear()
+
+    @property
+    @lru_cache(maxsize=None)
+    def _grd(self):
+        description = self.description
+        if self.crs is not None and self.crs.srs not in description:
+            description += f'; {self.crs.srs}'
+        return {
+            'nodes': self._nodes,
+            'elements': self._elements,
+            'description': description,
+        }
+
+    @property
+    @lru_cache(maxsize=None)
+    def _sms2dm(self):
+        description = self.description
+        if self.crs is not None:
+            description += f'; {self.crs.srs}'
+        return {
+            'ND': self._nodes,
+            'E3T': self._triangles,
+            'E4Q': self._quads,
+        }
+
+    @property
+    def _triangles(self):
+        return self.__triangles
+
+    @_triangles.setter
+    def _triangles(self, triangles):
+        if triangles is None:
+            triangles = {}
+        if isinstance(triangles, np.ndarray):
+            triangles = {
+                id: tuple(e) for id, e in enumerate(triangles)
+            }
+        self._certify_input_geom('triangles', triangles)
+        self.__triangles = triangles
+
+    @property
+    def _quads(self):
+        return self.__quads
+
+    @_quads.setter
+    def _quads(self, quads):
+        if quads is None:
+            quads = {}
+        self._certify_input_geom('quads', quads)
+        self.__quads = quads
+
+    @property
+    @lru_cache(maxsize=None)
+    def _elements(self):
+        elements_id = list()
+        elements_id.extend(list(self._triangles.keys()))
+        elements_id.extend(list(self._quads.keys()))
+        elements_id = range(1, len(elements_id) + 1) \
+            if len(set(elements_id)) != len(elements_id) else elements_id
+        elements = list()
+        elements.extend(list(self._triangles.values()))
+        elements.extend(list(self._quads.values()))
+        elements = {
+            elements_id[i]: indexes for i, indexes in enumerate(elements)}
+        return elements
+
+    @property
+    @lru_cache(maxsize=None)
+    def _logger(self):
+        return logging.getLogger(__name__ + '.' + self.__class__.__name__)
