@@ -11,13 +11,14 @@ from mpl_toolkits.axes_grid1 import make_axes_locatable
 import numpy as np
 from shapely.geometry import LineString, mapping
 
-from adcircpy.forcing._bctypes import _BoundaryCondition
-from adcircpy.forcing.winds._base import _WindForcing
-from adcircpy.mesh import _figures as fig, grd, sms2dm
-from adcircpy.mesh._base import _EuclideanMesh2D
+from adcircpy.forcing.bctypes import BoundaryCondition
+from adcircpy.forcing.waves import WaveForcing
+from adcircpy.forcing.winds.base import WindForcing
+from adcircpy.mesh import figures as fig, grd, sms2dm
+from adcircpy.mesh.base import EuclideanMesh2D
 
 
-class AdcircMesh(_EuclideanMesh2D):
+class AdcircMesh(EuclideanMesh2D):
     """
     Class that represents the unstructured planar mesh used by ADCIRC.
     """
@@ -33,10 +34,10 @@ class AdcircMesh(_EuclideanMesh2D):
         self._nodes = nodes
         self._elements = elements
         super().__init__(**grd.euclidean_mesh({
-            'nodes': self._nodes,
-            'elements': self._elements,
-            'description': description,
-            'crs': crs
+                'nodes': self._nodes,
+                'elements': self._elements,
+                'description': description,
+                'crs': crs
         }))
         self._boundaries = boundaries
 
@@ -58,8 +59,8 @@ class AdcircMesh(_EuclideanMesh2D):
         for idx in np.asarray(indexes).flatten():
             assert idx in self.node_index.keys(), msg
         self.__boundaries[ibtype][id] = {
-            'indexes': indexes,
-            **properties
+                'indexes': indexes,
+                **properties
         }
 
     def delete_boundary_type(self, ibtype):
@@ -79,24 +80,24 @@ class AdcircMesh(_EuclideanMesh2D):
                 driver='ESRI Shapefile',
                 crs=self.crs.srs,
                 schema={
-                    'geometry': 'LineString',
-                    'properties': {
-                        'id': 'int',
-                        'ibtype': 'str',
-                        'bnd_id': 'str'
-                    }}) as dst:
+                        'geometry': 'LineString',
+                        'properties': {
+                                'id': 'int',
+                                'ibtype': 'str',
+                                'bnd_id': 'str'
+                        }}) as dst:
             _cnt = 0
             for ibtype, bnds in self.boundaries.items():
                 for id, bnd in bnds.items():
                     idxs = list(map(self.get_node_index, bnd['indexes']))
                     linear_ring = LineString(self.xy[idxs].tolist())
                     dst.write({
-                        "geometry": mapping(linear_ring),
-                        "properties": {
-                            "id": _cnt,
-                            "ibtype": ibtype,
-                            "bnd_id": f"{ibtype}:{id}"
-                        }
+                            "geometry": mapping(linear_ring),
+                            "properties": {
+                                    "id": _cnt,
+                                    "ibtype": ibtype,
+                                    "bnd_id": f"{ibtype}:{id}"
+                            }
                     })
                     _cnt += 1
 
@@ -165,15 +166,17 @@ class AdcircMesh(_EuclideanMesh2D):
             self.set_boundary_data(interior_ibtype, bnd_id, data)
 
     def add_forcing(self, forcing, id=None):
-        if isinstance(forcing, _BoundaryCondition):
+        if isinstance(forcing, BoundaryCondition):
             if id is None:
                 for i in range(len(self.open_boundaries)):
                     self.add_forcing(forcing, i)
             else:
                 self._boundary_forcing[forcing.btype]["bnd_ids"].add(id)
                 self._boundary_forcing[forcing.btype].update({"obj": forcing})
-        elif isinstance(forcing, _WindForcing):
+        elif isinstance(forcing, WindForcing):
             self._surface_forcing.update({'imetype': forcing})
+        elif isinstance(forcing, WaveForcing):
+            self._surface_forcing.update({'iwrtype': forcing})
         else:
             msg = f"Unrecognized forcing type {forcing}."
             raise Exception(msg)
@@ -190,7 +193,7 @@ class AdcircMesh(_EuclideanMesh2D):
                                  f'{name}: attribute already exists.')
         else:
             self.add_attribute(
-                name, units=units, coldstart=False, hotstart=False)
+                    name, units=units, coldstart=False, hotstart=False)
             self._nodal_attribute_collection[name] = None
 
     def set_nodal_attribute(
@@ -207,9 +210,9 @@ class AdcircMesh(_EuclideanMesh2D):
         assert isinstance(coldstart, bool)
         assert isinstance(hotstart, bool)
         properties = {
-            'units': self._attributes[attribute_name]['units'],
-            'coldstart': coldstart,
-            'hotstart': hotstart
+                'units': self._attributes[attribute_name]['units'],
+                'coldstart': coldstart,
+                'hotstart': hotstart
         }
         self.set_attribute(attribute_name, values, **properties)
 
@@ -269,10 +272,10 @@ class AdcircMesh(_EuclideanMesh2D):
             _attr = self.get_attribute(name).copy()
             if _attr['values'].ndim == 1:  # rewrite as column major
                 _attr['values'] = _attr['values'].reshape(
-                    (_attr['values'].shape[0], 1))
+                        (_attr['values'].shape[0], 1))
             _attr['defaults'] = mode_rows(_attr['values'])
             _attr['non_default_indexes'] = np.where(
-                (_attr['values'] != _attr['defaults']).all(axis=1))[0]
+                    (_attr['values'] != _attr['defaults']).all(axis=1))[0]
             self._nodal_attribute_collection[name] = _attr
         return self._nodal_attribute_collection[name]
 
@@ -294,7 +297,7 @@ class AdcircMesh(_EuclideanMesh2D):
             return False
 
     def import_nodal_attributes(self, fort13):
-        fort13 = self._parse_fort13(fort13)
+        fort13 = parse_fort13(fort13)
         if fort13.pop('NumOfNodes') != len(self.node_id):
             raise Exception('fort.13 file does not match the mesh.')
         self._AGRID = fort13.pop('AGRID')
@@ -303,15 +306,15 @@ class AdcircMesh(_EuclideanMesh2D):
             if values.ndim == 1:
                 values = values.reshape((values.shape[0], 1))
             full_values = np.full(
-                (self.values.size,
-                 np.asarray(data['default_values']).flatten().size),
-                np.nan)
+                    (self.values.size,
+                     np.asarray(data['defaults']).flatten().size),
+                    np.nan)
             for i, idx in enumerate(data['indexes']):
                 for j, value in enumerate(values[i, :].tolist()):
                     full_values[idx, j] = value
             idxs = np.where(np.isnan(full_values).all(axis=1))[0]
             for idx in idxs:
-                for i, value in enumerate(data['default_values']):
+                for i, value in enumerate(data['defaults']):
                     full_values[idx, i] = value
             # converts from column major to row major, leave it column major.
             # if full_values.shape[1] == 1:
@@ -357,12 +360,12 @@ class AdcircMesh(_EuclideanMesh2D):
         if 'primitive_weighting_in_continuity_equation' \
                 not in self.get_nodal_attribute_names():
             self.add_nodal_attribute(
-                'primitive_weighting_in_continuity_equation',
-                'unitless'
+                    'primitive_weighting_in_continuity_equation',
+                    'unitless'
             )
         self.set_nodal_attribute(
-            'primitive_weighting_in_continuity_equation',
-            values
+                'primitive_weighting_in_continuity_equation',
+                values
         )
 
     def write_fort14(self, path, overwrite=False):
@@ -411,9 +414,9 @@ class AdcircMesh(_EuclideanMesh2D):
 
         # precompute distances
         distances = np.sqrt(
-            (self.x[edges[:, 0]] - self.x[edges[:, 1]]) ** 2
-            +
-            (self.y[edges[:, 0]] - self.y[edges[:, 1]]) ** 2
+                (self.x[edges[:, 0]] - self.x[edges[:, 1]]) ** 2
+                +
+                (self.y[edges[:, 0]] - self.y[edges[:, 1]]) ** 2
         )
         dz = np.abs(ffun[edges[:, 0]] - ffun[edges[:, 1]])
 
@@ -422,10 +425,10 @@ class AdcircMesh(_EuclideanMesh2D):
             _active_eges = np.zeros(edges.shape[0], dtype=bool)
             _active_eges[idxs] = True
             active_edges = edges[np.where(
-                np.logical_and(
-                    np.divide(dz, distances) > dfdx,
-                    _active_eges
-                ))]
+                    np.logical_and(
+                            np.divide(dz, distances) > dfdx,
+                            _active_eges
+                    ))]
             return active_edges
 
         def get_active_edges_traditional():
@@ -442,9 +445,11 @@ class AdcircMesh(_EuclideanMesh2D):
         while len(active_edges) > 0:
             cnt += 1
             active_distances = np.sqrt(
-                (self.x[active_edges[:, 0]] - self.x[active_edges[:, 1]]) ** 2
-                +
-                (self.y[active_edges[:, 0]] - self.y[active_edges[:, 1]]) ** 2
+                    (self.x[active_edges[:, 0]] - self.x[
+                        active_edges[:, 1]]) ** 2
+                    +
+                    (self.y[active_edges[:, 0]] - self.y[
+                        active_edges[:, 1]]) ** 2
             )
             for i, (p0, p1) in enumerate(active_edges):
                 z0, z1 = ffun[p0], ffun[p1]
@@ -475,10 +480,10 @@ class AdcircMesh(_EuclideanMesh2D):
                 idx = cnt_table.index(min(cnt_table))
                 if idx > 0:
                     self.limgrad(
-                        dfdx,
-                        idx,
-                        verbose=verbose,
-                        minimize=False
+                            dfdx,
+                            idx,
+                            verbose=verbose,
+                            minimize=False
                     )
 
         if 'original_crs' in locals():
@@ -489,7 +494,7 @@ class AdcircMesh(_EuclideanMesh2D):
     @fig._figure
     def make_plot(
             self,
-            axes: pyplot.Axes = None,
+            axes=None,
             vmin: float = None,
             vmax: float = None,
             show: bool = False,
@@ -499,6 +504,9 @@ class AdcircMesh(_EuclideanMesh2D):
             cbar_label: str = None,
             **kwargs
     ):
+        if axes is None:
+            _fig = plt.figure()
+            axes = _fig.add_subplot(111)
         if vmin is None:
             vmin = np.min(self.values)
         if vmax is None:
@@ -508,11 +516,11 @@ class AdcircMesh(_EuclideanMesh2D):
         levels = kwargs.pop('levels')
         if vmin != vmax:
             self.tricontourf(
-                axes=axes,
-                levels=levels,
-                vmin=vmin,
-                vmax=vmax,
-                **kwargs
+                    axes=axes,
+                    levels=levels,
+                    vmin=vmin,
+                    vmax=vmax,
+                    **kwargs
             )
         else:
             self.tripcolor(axes=axes, **kwargs)
@@ -528,16 +536,14 @@ class AdcircMesh(_EuclideanMesh2D):
         divider = make_axes_locatable(axes)
         cax = divider.append_axes("bottom", size="2%", pad=0.5)
         cbar = plt.colorbar(
-            mappable,
-            cax=cax,
-            orientation='horizontal'
+                mappable,
+                cax=cax,
+                orientation='horizontal'
         )
         cbar.set_ticks([vmin, vmax])
         cbar.set_ticklabels([np.around(vmin, 2), np.around(vmax, 2)])
         if cbar_label is not None:
             cbar.set_label(cbar_label)
-        if show:
-            pyplot.show()
         return axes
 
     @fig._figure
@@ -553,16 +559,16 @@ class AdcircMesh(_EuclideanMesh2D):
     ):
 
         boundary = list(map(
-            self.get_node_index,
-            self.boundaries[ibtype][id]['indexes']
+                self.get_node_index,
+                self.boundaries[ibtype][id]['indexes']
         ))
         p = axes.plot(self.x[boundary], self.y[boundary], **kwargs)
         if tags:
             axes.text(
-                self.x[boundary[len(boundary) // 2]],
-                self.y[boundary[len(boundary) // 2]],
-                f"ibtype={ibtype}\nid={id}",
-                color=p[-1].get_color()
+                    self.x[boundary[len(boundary) // 2]],
+                    self.y[boundary[len(boundary) // 2]],
+                    f"ibtype={ibtype}\nid={id}",
+                    color=p[-1].get_color()
             )
         if show:
             pyplot.show()
@@ -613,23 +619,23 @@ class AdcircMesh(_EuclideanMesh2D):
                 f += f'{idx + 1:d}\n'
         # count remaining boundaries
         num_remaining_boundaries = sum((
-            len(self.land_boundaries),
-            len(self.inner_boundaries),
-            len(self.inflow_boundaries),
-            len(self.outflow_boundaries),
-            len(self.weir_boundaries),
-            len(self.culvert_boundaries)
+                len(self.land_boundaries),
+                len(self.inner_boundaries),
+                len(self.inflow_boundaries),
+                len(self.outflow_boundaries),
+                len(self.weir_boundaries),
+                len(self.culvert_boundaries)
         ))
         f += f'{num_remaining_boundaries:d} '
         f += '! total number of non-ocean boundaries\n'
         # count total remaining boundary simplices
         num_remaining_boundary_simplices = int(sum((
-            np.sum([len(x) for x in self.land_boundaries]),
-            np.sum([len(x) for x in self.inner_boundaries]),
-            np.sum([len(x) for x in self.inflow_boundaries]),
-            np.sum([len(x) for x in self.outflow_boundaries]),
-            np.sum([2 * len(x) for x in self.weir_boundaries]),
-            np.sum([2 * len(x) for x in self.culvert_boundaries])
+                np.sum([len(x) for x in self.land_boundaries]),
+                np.sum([len(x) for x in self.inner_boundaries]),
+                np.sum([len(x) for x in self.inflow_boundaries]),
+                np.sum([len(x) for x in self.outflow_boundaries]),
+                np.sum([2 * len(x) for x in self.weir_boundaries]),
+                np.sum([2 * len(x) for x in self.culvert_boundaries])
         )))
 
         f += f'{num_remaining_boundary_simplices:d} '
@@ -730,12 +736,12 @@ class AdcircMesh(_EuclideanMesh2D):
     @property
     def quadratic_friction_coefficient_at_sea_floor(self):
         return self.get_attribute(
-            "quadratic_friction_coefficient_at_sea_floor")
+                "quadratic_friction_coefficient_at_sea_floor")
 
     @property
     def surface_directional_effective_roughness_length(self):
         return self.get_attribute(
-            "surface_directional_effective_roughness_length")
+                "surface_directional_effective_roughness_length")
 
     @property
     def surface_canopy_coefficient(self):
@@ -775,7 +781,7 @@ class AdcircMesh(_EuclideanMesh2D):
     @property
     def average_horizontal_eddy_viscosity_in_sea_water_wrt_depth(self):
         return self.get_attribute(
-            "average_horizontal_eddy_viscosity_in_sea_water_wrt_depth")
+                "average_horizontal_eddy_viscosity_in_sea_water_wrt_depth")
 
     @property
     def elemental_slope_limiter(self):
@@ -810,8 +816,8 @@ class AdcircMesh(_EuclideanMesh2D):
         for id in self._open_boundaries:
 
             indexes = list(map(
-                self.get_node_index,
-                self.boundaries[None][id]['indexes']))
+                    self.get_node_index,
+                    self.boundaries[None][id]['indexes']))
 
             def iettype(id):
                 if id in self._boundary_forcing['iettype']["bnd_ids"]:
@@ -839,13 +845,13 @@ class AdcircMesh(_EuclideanMesh2D):
                 return 0
 
             open_boundaries[id].update({
-                'indexes': indexes,
-                'neta': len(indexes),
-                'iettype': iettype(id),
-                'ifltype': ifltype(id),
-                'itetype': itetype(id),
-                'isatype': isatype(id),
-                'itrtype': itrtype(id)
+                    'indexes': indexes,
+                    'neta': len(indexes),
+                    'iettype': iettype(id),
+                    'ifltype': ifltype(id),
+                    'itetype': itetype(id),
+                    'isatype': isatype(id),
+                    'itrtype': itrtype(id)
             })
         return open_boundaries
 
@@ -883,10 +889,10 @@ class AdcircMesh(_EuclideanMesh2D):
                     else:
                         properties = {}
                     self.set_boundary_data(
-                        ibtype,
-                        id,
-                        bnd['indexes'],
-                        **properties
+                            ibtype,
+                            id,
+                            bnd['indexes'],
+                            **properties
                     )
 
     @_boundaries.deleter
@@ -902,8 +908,9 @@ class AdcircMesh(_EuclideanMesh2D):
     def _nodes(self, nodes):
         assert isinstance(nodes, dict)
         assert all(
-            len(node) == 2 and all(type(value) is float for value in node[0])
-            for node in nodes.values())
+                len(node) == 2 and all(
+                        type(value) is float for value in node[0])
+                for node in nodes.values())
         self.__nodes = nodes
 
     @property
@@ -934,19 +941,56 @@ class AdcircMesh(_EuclideanMesh2D):
     @lru_cache(maxsize=None)
     def _boundary_forcing(self):
         return {
-            'iettype': {"obj": None, "bnd_ids": set()},  # eta
-            'ifltype': {"obj": None, "bnd_ids": set()},  # flow
-            'itetype': {"obj": None, "bnd_ids": set()},  # temp
-            'isatype': {"obj": None, "bnd_ids": set()},  # salinity
-            'itrtype': {"obj": None, "bnd_ids": set()},  # tracer
-            'iwrtype': {"obj": None, "bnd_ids": set()},  # waves
+                'iettype': {"obj": None, "bnd_ids": set()},  # eta
+                'ifltype': {"obj": None, "bnd_ids": set()},  # flow
+                'itetype': {"obj": None, "bnd_ids": set()},  # temp
+                'isatype': {"obj": None, "bnd_ids": set()},  # salinity
+                'itrtype': {"obj": None, "bnd_ids": set()},  # tracer
+                'iwrtype': {"obj": None, "bnd_ids": set()},  # waves
         }
 
     @property
     @lru_cache(maxsize=None)
     def _surface_forcing(self):
         return {
-            'imetype': None,  # meteorological,
-            'iwrtype': None,
-            'isrtype': None,  # rain
+                'imetype': None,  # meteorological,
+                'iwrtype': None,
+                'isrtype': None,  # rain
         }
+
+
+def parse_fort13(path):
+    fort13 = {}
+    with open(path, 'r') as f:
+        fort13['AGRID'] = f.readline().strip()
+        fort13['NumOfNodes'] = NP = int(f.readline().split()[0])
+        NAttr = int(f.readline().split()[0])
+        i = 0
+        while i < NAttr:
+            attribute_name = f.readline().strip()
+            units = f.readline().strip()
+            if units == '1':
+                units = 'unitless'
+            f.readline()
+            defaults = [float(x) for x in f.readline().split()]
+            fort13[attribute_name] = {'units': units,
+                                      'defaults': defaults,
+                                      'indexes': []}
+            i += 1
+        for i in range(NAttr):
+            attribute_name = f.readline().strip()
+            numOfNodes = int(f.readline())
+            values = np.zeros((NP, len(fort13[attribute_name]['defaults'])))
+            values[:] = np.nan
+            j = 0
+            while j < numOfNodes:
+                str = f.readline().split()
+                index = int(str[0]) - 1
+                fort13[attribute_name]['indexes'].append(index)
+                node_values = [float(x) for x in str[1:]]
+                values[index, :] = node_values
+                j += 1
+            values[np.where(np.isnan(values[:, 0])), :] = \
+                fort13[attribute_name]['defaults']
+            fort13[attribute_name]['values'] = values
+        return fort13
